@@ -1,4 +1,4 @@
-# Copyright 2016-2021 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2022 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -6,8 +6,9 @@
 import abc
 
 import reframe.core.fields as fields
+import reframe.core.warnings as warn
+import reframe.utility as util
 import reframe.utility.typecheck as typ
-from reframe.core.exceptions import ContainerError
 
 
 _STAGEDIR_MOUNT = '/rfm_workdir'
@@ -80,24 +81,18 @@ class ContainerPlatform(abc.ABC):
     #: :default: ``[]``
     options = fields.TypedField(typ.List[str])
 
-    _workdir = fields.TypedField(str, type(None))
     #: The working directory of ReFrame inside the container.
     #:
     #: This is the directory where the test's stage directory is mounted inside
     #: the container. This directory is always mounted regardless if
     #: :attr:`mount_points` is set or not.
     #:
-    #: .. deprecated:: 3.5
-    #:    Please use the `options` field to set the working directory.
-    #:
     #: :type: :class:`str`
     #: :default: ``/rfm_workdir``
-    workdir = fields.DeprecatedField(
-        _workdir,
-        'The `workdir` field is deprecated, please use the `options` field to '
-        'set the container working directory',
-        fields.DeprecatedField.OP_SET, from_version='3.5.0'
-    )
+    #:
+    #: .. versionchanged:: 3.12.0
+    #:    This attribute is no more deprecated.
+    workdir = fields.TypedField(str, type(None))
 
     def __init__(self):
         self.image = None
@@ -106,8 +101,8 @@ class ContainerPlatform(abc.ABC):
         # NOTE: Here we set the target fields directly to avoid the deprecation
         # warnings
         self._commands = []
-        self._workdir = _STAGEDIR_MOUNT
 
+        self.workdir = _STAGEDIR_MOUNT
         self.mount_points  = []
         self.options = []
         self.pull_image = True
@@ -143,12 +138,37 @@ class ContainerPlatform(abc.ABC):
         :arg stagedir: The stage directory of the test.
         '''
 
-    def validate(self):
-        if self.image is None:
-            raise ContainerError('no image specified')
+    @classmethod
+    def create(cls, name):
+        '''Factory method to create a new container by name.'''
+        name = name.capitalize()
+        try:
+            return globals()[name]()
+        except KeyError:
+            raise ValueError(f'unknown container platform: {name}') from None
+
+    @classmethod
+    def create_from(cls, name, other):
+        new = cls.create(name)
+        new.image = other.image
+        new.command = other.command
+        new.mount_points = other.mount_points
+        new.options = other.options
+        new.pull_image = other.pull_image
+        new.workdir = other.workdir
+
+        # Update deprecated fields
+        with warn.suppress_deprecations():
+            new.commands = other.commands
+
+        return new
+
+    @property
+    def name(self):
+        return type(self).__name__
 
     def __str__(self):
-        return type(self).__name__
+        return self.name
 
     def __rfm_json_encode__(self):
         return str(self)
@@ -165,15 +185,17 @@ class Docker(ContainerPlatform):
         super().launch_command(stagedir)
         mount_points = self.mount_points + [(stagedir, _STAGEDIR_MOUNT)]
         run_opts = [f'-v "{mp[0]}":"{mp[1]}"' for mp in mount_points]
-        run_opts += self.options
+        if self.workdir:
+            run_opts.append(f'-w {self.workdir}')
 
+        run_opts += self.options
         if self.command:
             return (f'docker run --rm {" ".join(run_opts)} '
                     f'{self.image} {self.command}')
 
         if self.commands:
             return (f"docker run --rm {' '.join(run_opts)} {self.image} "
-                    f"bash -c 'cd {self.workdir}; {'; '.join(self.commands)}'")
+                    f"bash -c '{'; '.join(self.commands)}'")
 
         return f'docker run --rm {" ".join(run_opts)} {self.image}'
 
@@ -197,7 +219,8 @@ class Sarus(ContainerPlatform):
         # The format that Sarus uses to call the images is
         # <reposerver>/<user>/<image>:<tag>. If an image was loaded
         # locally from a tar file, the <reposerver> is 'load'.
-        if not self.pull_image or self.image.startswith('load/'):
+        if (not self.pull_image or not self.image or
+            self.image.startswith('load/')):
             return []
         else:
             return [f'{self._command} pull {self.image}']
@@ -210,15 +233,17 @@ class Sarus(ContainerPlatform):
         if self.with_mpi:
             run_opts.append('--mpi')
 
-        run_opts += self.options
+        if self.workdir:
+            run_opts.append(f'-w {self.workdir}')
 
+        run_opts += self.options
         if self.command:
             return (f'{self._command} run {" ".join(run_opts)} {self.image} '
                     f'{self.command}')
 
         if self.commands:
             return (f"{self._command} run {' '.join(run_opts)} {self.image} "
-                    f"bash -c 'cd {self.workdir}; {'; '.join(self.commands)}'")
+                    f"bash -c '{'; '.join(self.commands)}'")
 
         return f'{self._command} run {" ".join(run_opts)} {self.image}'
 
@@ -232,6 +257,12 @@ class Shifter(Sarus):
         super().__init__()
         self._command = 'shifter'
 
+    def launch_command(self, stagedir):
+        # Temporarily change `workdir`, since Sarus and Shifter have otherwise
+        # the same interface
+        with util.temp_setattr(self, 'workdir', None):
+            return super().launch_command(stagedir)
+
 
 class Singularity(ContainerPlatform):
     '''Container platform backend for running containers with `Singularity
@@ -257,6 +288,9 @@ class Singularity(ContainerPlatform):
         if self.with_cuda:
             run_opts.append('--nv')
 
+        if self.workdir:
+            run_opts.append(f'-W {self.workdir}')
+
         run_opts += self.options
         if self.command:
             return (f'singularity exec {" ".join(run_opts)} '
@@ -264,7 +298,7 @@ class Singularity(ContainerPlatform):
 
         if self.commands:
             return (f"singularity exec {' '.join(run_opts)} {self.image} "
-                    f"bash -c 'cd {self.workdir}; {'; '.join(self.commands)}'")
+                    f"bash -c '{'; '.join(self.commands)}'")
 
         return f'singularity run {" ".join(run_opts)} {self.image}'
 
@@ -275,10 +309,6 @@ class ContainerPlatformField(fields.TypedField):
 
     def __set__(self, obj, value):
         if isinstance(value, str):
-            try:
-                value = globals()[value]()
-            except KeyError:
-                raise ValueError(
-                    f'unknown container platform: {value}') from None
+            value = ContainerPlatform.create(value)
 
         super().__set__(obj, value)
