@@ -119,7 +119,7 @@ class FixtureRegistry:
         # Store the system name for name-mangling purposes
         self._sys_name = runtime.runtime().system.name
 
-    def add(self, fixture, variant_num, parent_name, partitions, prog_envs):
+    def add(self, fixture, variant_num, parent_test):
         '''Register a fixture.
 
         This method mangles the fixture name, ensuring that different fixture
@@ -161,11 +161,7 @@ class FixtureRegistry:
 
         :param fixture: An instance of :class:`TestFixture`.
         :param variant_num: The variant index for the given ``fixture``.
-        :param parent_name: The full name of the parent test. This argument
-            is used to mangle the fixture name for those with a ``'test'``
-            scope, such that the fixture is private to its parent test.
-        :param partitions: The system partitions supported by the parent test.
-        :param prog_envs: The valid programming environments from the parent.
+        :param parent_test: The parent test.
         '''
 
         cls = fixture.cls
@@ -187,51 +183,65 @@ class FixtureRegistry:
             fname += vname
 
         # Select only the valid partitions
-        valid_partitions = self._filter_valid_partitions(partitions)
+        try:
+            valid_sysenv = runtime.valid_sysenv_comb(
+                parent_test.valid_systems,
+                parent_test.valid_prog_environs
+            )
+        except AttributeError as e:
+            msg = e.args[0] + f' in test {parent_test.display_name!r}'
+            raise ReframeSyntaxError(msg) from None
 
-        # Return if not any valid partition
-        if not valid_partitions:
+        # Return if there are no valid system/environment combinations
+        if not valid_sysenv:
             return []
 
         # Register the fixture
         if scope == 'session':
             # The name is mangled with the system name
-            # Select a valid environment supported by a partition
-            for part in valid_partitions:
-                valid_envs = self._filter_valid_environs(part, prog_envs)
-                if valid_envs:
+            # Pick the first valid system/environment combination
+            pname, ename = None, None
+            for part, environs in valid_sysenv.items():
+                pname = part.fullname
+                for env in environs:
+                    ename = env.name
                     break
-            else:
+
+            if ename is None:
+                # No valid environments found
                 return []
 
             # Register the fixture
-            fixt_data = FixtureData(variant_num, [valid_envs[0]], [part],
+            fixt_data = FixtureData(variant_num, [ename], [pname],
                                     variables, scope, self._sys_name)
             name = f'{cls.__name__}_{fixt_data.mashup()}'
             self._registry[cls][name] = fixt_data
             reg_names.append(name)
         elif scope == 'partition':
-            for part in valid_partitions:
+            for part, environs in valid_sysenv.items():
                 # The mangled name contains the full partition name
-
                 # Select an environment supported by the partition
-                valid_envs = self._filter_valid_environs(part, prog_envs)
-                if not valid_envs:
+                pname = part.fullname
+                try:
+                    ename = environs[0].name
+                except IndexError:
                     continue
 
                 # Register the fixture
-                fixt_data = FixtureData(variant_num, [valid_envs[0]], [part],
-                                        variables, scope, part)
+                fixt_data = FixtureData(variant_num, [ename], [pname],
+                                        variables, scope, pname)
                 name = f'{cls.__name__}_{fixt_data.mashup()}'
                 self._registry[cls][name] = fixt_data
                 reg_names.append(name)
         elif scope == 'environment':
-            for part in valid_partitions:
-                for env in self._filter_valid_environs(part, prog_envs):
+            for part, environs in valid_sysenv.items():
+                for env in environs:
                     # The mangled name contains the full part and env names
                     # Register the fixture
-                    fixt_data = FixtureData(variant_num, [env], [part],
-                                            variables, scope, f'{part}+{env}')
+                    pname, ename = part.fullname, env.name
+                    fixt_data = FixtureData(variant_num, [ename], [pname],
+                                            variables, scope,
+                                            f'{pname}+{ename}')
                     name = f'{cls.__name__}_{fixt_data.mashup()}'
                     self._registry[cls][name] = fixt_data
                     reg_names.append(name)
@@ -239,9 +249,10 @@ class FixtureRegistry:
             # The mangled name contains the parent test name.
 
             # Register the fixture
-            fixt_data = FixtureData(variant_num, list(prog_envs),
-                                    list(valid_partitions),
-                                    variables, scope, parent_name)
+            fixt_data = FixtureData(variant_num,
+                                    list(parent_test.valid_prog_environs),
+                                    list(parent_test.valid_systems),
+                                    variables, scope, parent_test.unique_name)
             name = f'{cls.__name__}_{fixt_data.mashup()}'
             self._registry[cls][name] = fixt_data
             reg_names.append(name)
@@ -348,54 +359,411 @@ class FixtureRegistry:
 
 
 class TestFixture:
-    '''Regression test fixture class.
+    '''Insert a new fixture in the current test.
 
-    A fixture is a regression test that generates a resource that must exist
-    before the parent test is executed. A fixture is a class that derives from
-    the :class:`reframe.core.pipeline.RegressionTest` class and serves as a
-    building block to compose a more complex test structure. Since fixtures are
-    full ReFrame tests on their own, a fixture can have multiple fixtures, and
-    so on; building a directed acyclic graph.
+    A fixture is a regression test that creates, prepares and/or manages a
+    resource for another regression test. Fixtures may contain other fixtures
+    and so on, forming a directed acyclic graph. A parent fixture (or a
+    regular regression test) requires the resources managed by its child
+    fixtures in order to run, and it may only access these fixture resources
+    after its ``setup`` pipeline stage. The execution of parent fixtures is
+    postponed until all their respective children have completed execution.
+    However, the destruction of the resources managed by a fixture occurs in
+    reverse order, only after all the parent fixtures have been destroyed.
+    This destruction of resources takes place during the ``cleanup`` pipeline
+    stage of the regression test. Fixtures must not define the members
+    :attr:`~reframe.core.pipeline.RegressionTest.valid_systems` and
+    :attr:`~reframe.core.pipeline.RegressionTest.valid_prog_environs`. These
+    variables are defined based on the values specified in the parent test,
+    ensuring that the fixture runs with a suitable system partition and
+    programming environment combination. A fixture's
+    :attr:`~reframe.core.pipeline.RegressionTest.name` attribute may be
+    internally mangled depending on the arguments passed during the fixture
+    declaration. Hence, manually setting or modifying the
+    :attr:`~reframe.core.pipeline.RegressionTest.name` attribute in the
+    fixture class is disallowed, and breaking this restriction will result in
+    undefined behavior.
 
-    However, a given fixture may be shared by multiple regression tests that
-    need the same resource. This can be achieved by setting the appropriate
-    scope level on which the fixture should be shared. By default, fixtures
-    are registered with the ``'test'`` scope, which makes each fixture
-    `private` to each of the parent tests. Hence, if all fixtures use this
-    scope, the resulting fixture hierarchy can be thought of multiple
-    independent branches that emanate from each root regression test. On the
-    other hand, setting a more relaxed scope that allows resource sharing
-    across different regression tests will effectively interconnect the
-    fixture branches that share a resource.
+    .. warning::
+       The fixture name mangling is considered an internal framework mechanism
+       and it may change in future versions without any notice. Users must not
+       express any logic in their tests that relies on a given fixture name
+       mangling scheme.
 
-    From a more to less restrictive scope, the valid scopes are ``'test'``,
-    ``'environment'``, ``'partition'`` and ``'session'``. Fixtures with
-    a scope set to either ``'partition'`` or ``'session'`` must derive from
-    the :class:`reframe.core.pipeline.RunOnlyRegressionTest` class, since the
-    generated resource must not depend on the programming environment. Fixtures
-    with scopes set to either ``'environment'`` or ``'test'`` can derive from
-    any derived class from :class:`reframe.core.pipeline.RegressionTest`.
 
-    Fixtures may be parameterized, where a regression test that uses a
-    parameterized fixture is by extension a parameterized test. Hence, the
-    number of test variants of a test will depend on the test parameters and
-    the parameters of each of the fixtures that compose the parent test. Each
-    possible parameter-fixture combination has a unique `variant number`, which
-    is an index in the range from ``[0, N)``, where `N` is the total number of
-    test variants. This is the default behaviour and it is achieved when the
-    action argument is set to ``'fork'``. On the other hand, if this argument
-    is set to a ``'join'`` action, the parent test will reduce all the fixture
+    By default, the resources managed by a fixture are private to the parent
+    test. However, it is possible to share these resources across different
+    tests by passing the appropriate fixture ``scope`` argument. The different
+    scope levels are independent from each other and a fixture only executes
+    once per scope, where all the tests that belong to that same scope may use
+    the same resources managed by a given fixture instance. The available
+    scopes are:
+
+      - **session**: This scope encloses all the tests and fixtures that run
+        in the full ReFrame session. This may include tests that use different
+        system partition and programming environment combinations. The fixture
+        class must derive from
+        :class:`~reframe.core.pipeline.RunOnlyRegressionTest` to avoid any
+        implicit dependencies on the partition or the programming environment
+        used.
+
+      - **partition**: This scope spans across a single system partition. This
+        may include different tests that run on the same partition but use
+        different programming environments. Fixtures with this scope must be
+        independent of the programming environment, which restricts the
+        fixture class to derive from
+        :class:`~reframe.core.pipeline.RunOnlyRegressionTest`.
+
+      - **environment**: The extent of this scope covers a single combination
+        of system partition and programming environment. Since the fixture is
+        guaranteed to have the same partition and programming environment as
+        the parent test, the fixture class can be any derived class from
+        :class:`~reframe.core.pipeline.RegressionTest`. * **test**: This scope
+        covers a single instance of the parent test, where the resources
+        provided by the fixture are exclusive to each parent test instance.
+        The fixture class can be any derived class from
+        :class:`~reframe.core.pipeline.RegressionTest`.
+
+    Rather than specifying the scope at the fixture class definition, ReFrame
+    fixtures set the scope level from the consumer side (i.e. when used by
+    another test or fixture). A test may declare multiple fixtures using the
+    same class, where fixtures with different scopes are guaranteed to point
+    to different instances of the fixture class. On the other hand, when two
+    or more fixtures use the same fixture class and have the same scope, these
+    different fixtures will point to the same underlying resource if the
+    fixtures refer to the same :ref:`variant<test-variants>` of the fixture
+    class. The example below illustrates the different fixture scope usages:
+
+    .. code:: python
+
+       class MyFixture(rfm.RunOnlyRegressionTest):
+          my_var = variable(int, value=1)
+          ...
+
+
+       @rfm.simple_test
+       class TestA(rfm.RegressionTest):
+           valid_systems = ['p1', 'p2']
+           valid_prog_environs = ['e1', 'e2']
+
+           # Fixture shared throughout the full session
+           f1 = fixture(MyFixture, scope='session')
+
+           # Fixture shared for each supported partition
+           f2 = fixture(MyFixture, scope='partition')
+
+           # Fixture shared for each supported part+environ
+           f3 = fixture(MyFixture, scope='environment')
+
+           # Fixture private evaluation of MyFixture
+           f4 = fixture(MyFixture, scope='test')
+           ...
+
+
+       @rfm.simple_test
+       class TestB(rfm.RegressionTest):
+           valid_systems = ['p1']
+           valid_prog_environs = ['e1']
+
+           # Another private instance of MyFixture
+           f1 = fixture(MyFixture, scope='test')
+
+           # Same as f3 in TestA for p1 + e1
+           f2 = fixture(MyFixture, scope='environment')
+
+           # Same as f1 in TestA
+           f3 = fixture(MyFixture, scope='session')
+           ...
+
+           @run_after('setup')
+           def access_fixture_resources(self):
+               # Dummy pipeline hook to illustrate fixture resource access
+               assert self.f1.my_var is not self.f2.my_var
+               assert self.f1.my_var is not self.f3.my_var
+
+
+    :class:`TestA` supports two different valid systems and another two valid
+    programming environments. Assuming that both environments are supported by
+    each of the system partitions ``'p1'`` and ``'p2'``, this test will
+    execute a total of four times. This test uses the very simple
+    :class:`MyFixture` fixture multiple times using different scopes, where
+    fixture ``f1`` (session scope) will be shared across the four test
+    instances, and fixture ``f4`` (test scope) will be executed once per test
+    instance. On the other hand, ``f2`` (partition scope) will run once per
+    partition supported by test :class:`TestA`, and the multiple per-partition
+    executions (i.e. for each programming environment) will share the same
+    underlying resource for ``f2``. Lastly, ``f3`` will run a total of four
+    times, which is once per partition and environment combination. This
+    simple :class:`TestA` shows how multiple instances from the same test can
+    share resources, but the real power behind fixtures is illustrated with
+    :class:`TestB`, where this resource sharing is extended across different
+    tests. For simplicity, :class:`TestB` only supports a single partition
+    ``'p1'`` and programming environment ``'e1'``, and similarly to
+    :class:`TestA`, ``f1`` (test scope) causes a private evaluation of the
+    fixture :class:`MyFixture`. However, the resources managed by fixtures
+    ``f2`` (environment scope) and ``f3`` (session scope) are shared with
+    :class:`Test1`.
+
+    Fixtures are treated by ReFrame as first-class ReFrame tests, which means
+    that these classes can use the same built-in functionalities as in regular
+    tests decorated with
+    :func:`@rfm.simple_test<reframe.core.decorators.simple_test>`. This
+    includes the :func:`~reframe.core.pipeline.RegressionMixin.parameter`
+    built-in, where fixtures may have more than one
+    :ref:`variant<test-variants>`. When this occurs, a parent test may select
+    to either treat a parameterized fixture as a test parameter, or instead,
+    to gather all the fixture variants from a single instance of the parent
+    test. In essence, fixtures implement `fork-join` model whose behavior may
+    be controlled through the ``action`` argument. This argument may be set to
+    one of the following options:
+
+      - **fork**: This option parameterizes the parent test as a function of
+        the fixture variants. The fixture handle will resolve to a single
+        instance of the fixture.
+
+      - **join**: This option gathers all the variants from a fixture into a
+        single instance of the parent test. The fixture handle will point to a
+        list containing all the fixture variants.
+
+    A test may declare multiple fixtures with different ``action`` options,
+    where the default ``action`` option is ``'fork'``. The example below
+    illustrates the behavior of these two different options.
+
+    .. code:: python
+
+       class ParamFix(rfm.RegressionTest):
+           p = parameter(range(5)) # A simple test parameter
+           ...
+
+
+       @rfm.simple_test
+       class TestC(rfm.RegressionTest):
+           # Parameterize TestC for each ParamFix variant
+           f = fixture(ParamFix, action='fork')
+           ...
+
+           @run_after('setup')
+           def access_fixture_resources(self):
+               print(self.f.p) # Prints the fixture's variant parameter value
+
+
+       @rfm.simple_test
+       class TestD(rfm.RegressionTest):
+           # Gather all fixture variants into a single test
+           f = fixture(ParamFix, action='join')
+           ...
+
+           @run_after('setup')
+           def reduce_range(self):
+               # Sum all the values of p for each fixture variant
+               res = functools.reduce(lambda x, y: x+y,
+                                      (fix.p for fix in self.f))
+               n = len(self.f)-1
+               assert res == (n*n + n)/2
+
+    Here :class:`ParamFix` is a simple fixture class with a single parameter.
+    When the test :class:`TestC` uses this fixture with a ``'fork'`` action,
+    the test is implicitly parameterized over each variant of
+    :class:`ParamFix`. Hence, when the :func:`access_fixture_resources`
+    post-setup hook accesses the fixture ``f``, it only access a single
+    instance of the :class:`ParamFix` fixture. On the other hand, when this
+    same fixture is used with a ``'join'`` action by :class:`TestD`, the test
+    is not parameterized and all the :class:`ParamFix` instances are gathered
+    into ``f`` as a list. Thus, the post-setup pipeline hook
+    :func:`reduce_range` can access all the fixture variants and compute a
+    reduction of the different ``p`` values.
+
+    When declaring a fixture, a parent test may select a subset of the fixture
+    variants through the ``variants`` argument. This variant selection can be
+    done by either passing an iterable containing valid variant indices (see
+    :ref:`test-variants` for further information on how the test variants are
+    indexed), or instead, passing a mapping with the parameter name (of the
+    fixture class) as keys and filtering functions as values. These filtering
+    functions are unary functions that return the value of a boolean
+    expression on the values of the specified parameter, and they all must
+    evaluate to :class:`True` for at least one of the fixture class variants.
+    See the example below for an illustration on how to filter-out fixture
     variants.
 
-    The variants from a given fixture to be used by the parent test can be
-    filtered out through the ``variants`` optional argument. This can either be
-    a list of the variant numbers to be used, or it can be a dictionary with
-    conditions on the parameter space of the fixture.
+    .. code:: python
 
-    Also, a fixture may set or update the default value of a test variable
-    by passing the appropriate key-value mapping as the ``variables`` argument.
+       class ComplexFixture(rfm.RegressionTest):
+           # A fixture with 400 different variants.
+           p0 = parameter(range(100))
+           p1 = parameter(['a', 'b', 'c', 'd'])
+           ...
+
+       @rfm.simple_test
+       class TestE(rfm.RegressionTest):
+           # Select the fixture variants with boolean conditions
+           foo = fixture(ComplexFixture,
+                         variants={'p0': lambda x: x<10,
+                                   'p1': lambda x: x=='d'})
+
+           # Select the fixture variants by index
+           bar = fixture(ComplexFixture, variants=range(300,310))
+           ...
+
+    A parent test may also specify the value of different variables in the
+    fixture class to be set before its instantiation. Each variable must have
+    been declared in the fixture class with the
+    :func:`~reframe.core.pipeline.RegressionMixin.variable` built-in,
+    otherwise it is silently ignored. This variable specification is
+    equivalent to deriving a new class from the fixture class, and setting
+    these variable values in the class body of a newly derived class.
+    Therefore, when fixture declarations use the same fixture class and pass
+    different values to the ``variables`` argument, the fixture class is
+    interpreted as a different class for each of these fixture declarations.
+    See the example below.
+
+    .. code:: python
+
+       class Fixture(rfm.RegressionTest):
+           v = variable(int, value=1)
+           ...
+
+       @rfm.simple_test
+       class TestF(rfm.RegressionTest):
+           foo = fixture(Fixture)
+           bar = fixture(Fixture, variables={'v':5})
+           baz = fixture(Fixture, variables={'v':10})
+           ...
+
+           @run_after('setup')
+           def print_fixture_variables(self):
+               print(self.foo.v) # Prints 1
+               print(self.bar.v) # Prints 5
+               print(self.baz.v) # Prints 10
+
+    The test :class:`TestF` declares the fixtures ``foo``, ``bar`` and ``baz``
+    using the same :class:`Fixture` class. If no variables were set in ``bar``
+    and ``baz``, this would result into the same fixture being declared
+    multiple times in the same scope (implicitly set to ``'test'``), which
+    would lead to a single instance of :class:`Fixture` being referred to by
+    ``foo``, ``bar`` and ``baz``. However, in this case ReFrame identifies
+    that the declared fixtures pass different values to the ``variables``
+    argument in the fixture declaration, and executes these three fixtures
+    separately.
+
+    .. note::
+       Mappings passed to the ``variables`` argument that define the same
+       class variables in different order are interpreted as the same value.
+       The two fixture declarations below are equivalent, and both ``foo`` and
+       ``bar`` will point to the same instance of the fixture class
+       :class:`MyResource`.
+
+       .. code:: python
+
+         foo = fixture(MyResource, variables={'a':1, 'b':2})
+         bar = fixture(MyResource, variables={'b':2, 'a':1})
+
+
+    **Early access to fixture objects**
+
+    The test instance represented by a fixture can be accessed fully from
+    within a test only after the setup stage. The reason for that is that
+    fixtures eventually translate into test dependencies and access to the
+    parent dependencies cannot happen before the this stage.
+
+    However, it is often useful, especially in the case of parameterized
+    fixtures, to be able to access the fixture parameters earlier, e.g., in a
+    post-init hook in order to properly set the
+    :attr:`~reframe.core.pipeline.RegressionTest.valid_systems` and
+    :attr:`~reframe.core.pipeline.RegressionTest.valid_prog_environs` of the
+    test. These attributes cannot be set later than the test's initialization
+    in order to have an effect.
+
+    For this reason, early access to fixture objects is allowed *only* for
+    retrieving their parameters.
+
+    .. code-block:: python
+
+       class Fixture(rfm.RegressionTest):
+           x = parameter([1, 2, 3])
+
+
+       class Test(rfm.RunOnlyRegressionTest):
+           foo = fixture(Fixture)
+           executable = './myexec'
+           valid_prog_environs = ['*']
+
+           @run_after('init')
+           def early_access(self):
+                # Only fixture parameters can be accessed here!
+                if self.foo.x == 1:
+                    self.valid_systems = ['sys1]
+                else:
+                    self.valid_systems = ['sys2']
+
+           @run_after('setup')
+           def normal_access(self):
+               # Any test attribute of the associated fixture test can be
+               # accessed here
+               self.executable_opts = [
+                   '-i', os.path.join(self.foo.stagedir, 'input.txt')
+               ]
+
+
+    During test initialization, ReFrame binds the :attr:`foo` name to a proxy
+    object that holds the parameterization of the target fixture. This proxy
+    object is recursive, so that if fixture :attr:`foo` contained another
+    fixture named :attr:`bar`, it would allow you to access any parameters of
+    that fixture with ``self.foo.bar.param``.
+
+    During the test setup stage, the :attr:`foo`'s binding changes and it is
+    now bound to the exact test instance that was executed for the target test
+    instance.
+
+    :param cls: A class derived from
+        :class:`~reframe.core.pipeline.RegressionTest` that manages a given
+        resource. The base from this class may be further restricted to other
+        derived classes of :class:`~reframe.core.pipeline.RegressionTest`
+        depending on the ``scope`` parameter.
+
+    :param scope: Sets the extent to which other regression tests may share
+        the resources managed by a fixture. The available scopes are, from
+        more to less restrictive, ``'test'``, ``'environment'``,
+        ``'partition'`` and ``'session'``. By default a fixture's scope is set
+        to ``'test'``, which makes the resource private to the test that uses
+        the fixture. This means that when multiple regression tests use the
+        same fixture class with a ``'test'`` scope, the fixture will run once
+        per regression test. When the scope is set to ``'environment'``, the
+        resources managed by the fixture are shared across all the tests that
+        use the fixture and run on the same system partition and use the same
+        programming environment. When the scope is set to ``'partition'``, the
+        resources managed by the fixture are shared instead across all the
+        tests that use the fixture and run on the same system partition.
+        Lastly, when the scope is set to ``'session'``, the resources managed
+        by the fixture are shared across the full ReFrame session. Fixtures
+        with either ``'partition'`` or ``'session'`` scopes may be shared
+        across different regression tests under different programming
+        environments, and for this reason, when using these two scopes, the
+        fixture class ``cls`` is required to derive from
+        :class:`~reframe.core.pipeline.RunOnlyRegressionTest`.
+
+    :param action: Set the behavior of a parameterized fixture to either
+        ``'fork'`` or ``'join'``. With a ``'fork'`` action, a parameterized
+        fixture effectively parameterizes the regression test. On the other
+        hand, a ``'join'`` action gathers all the fixture variants into the
+        same instance of the regression test. By default, the ``action``
+        parameter is set to ``'fork'``.
+
+    :param variants: Filter or sub-select a subset of the variants from a
+        parameterized fixture. This argument can be either an iterable with
+        the indices from the desired variants, or a mapping containing unary
+        functions that return the value of a boolean expression on the values
+        of a given parameter.
+
+    :param variables: Mapping to set the values of fixture's variables. The
+        variables are set after the fixture class has been created (i.e. after
+        the class body has executed) and before the fixture class is
+        instantiated.
+
+
+    .. versionadded:: 3.9.0
+    .. versionchanged:: 3.11.0
+       Allow early access of fixture objects.
 
-    :meta private:
     '''
 
     def __init__(self, cls, *, scope='test', action='fork', variants='all',
@@ -522,6 +890,16 @@ class TestFixture:
         return self._variables
 
 
+class FixtureProxy:
+    def __init__(self, fixture_info):
+        for k, v in fixture_info['params'].items():
+            setattr(self, k, v)
+
+        for k, v in fixture_info['fixtures'].items():
+            if not isinstance(v, tuple):
+                setattr(self, k, FixtureProxy(v))
+
+
 class FixtureSpace(namespaces.Namespace):
     '''Regression test fixture space.
 
@@ -619,18 +997,13 @@ class FixtureSpace(namespaces.Namespace):
         # Create the fixture registry
         obj._rfm_fixture_registry = FixtureRegistry()
 
-        # Prepare the partitions and prog_envs
-        part, prog_envs = self._expand_partitions_envs(obj)
-
         # Register the fixtures
         for name, fixture in self.fixtures.items():
             dep_names = []
             for variant in fixture_variants[name]:
                 # Register all the variants and track the fixture names
                 dep_names += obj._rfm_fixture_registry.add(fixture,
-                                                           variant,
-                                                           obj.name, part,
-                                                           prog_envs)
+                                                           variant, obj)
 
             # Add dependencies
             if fixture.scope == 'session':
@@ -644,38 +1017,6 @@ class FixtureSpace(namespaces.Namespace):
             for dep_name in dep_names:
                 obj.depends_on(dep_name, dep_kind)
 
-    def _expand_partitions_envs(self, obj):
-        '''Process the partitions and programming environs of the parent.'''
-
-        try:
-            part = tuple(obj.valid_systems)
-        except AttributeError:
-            raise ReframeSyntaxError(
-                f"'valid_systems' is undefined in test {obj.unique_name!r}"
-            )
-        else:
-            rt = runtime.runtime()
-            if '*' in part or rt.system.name in part:
-                part = tuple(p.fullname for p in rt.system.partitions)
-
-        try:
-            prog_envs = tuple(obj.valid_prog_environs)
-        except AttributeError:
-            raise ReframeSyntaxError(
-                f"'valid_prog_environs' is undefined "
-                f"in test {obj.unique_name!r}"
-            )
-        else:
-            if '*' in prog_envs:
-                all_pes = set()
-                for p in runtime.runtime().system.partitions:
-                    for e in p.environs:
-                        all_pes.add(e.name)
-
-                prog_envs = tuple(all_pes)
-
-        return part, prog_envs
-
     def __iter__(self):
         '''Walk through all index combinations for all fixtures.'''
         yield from self.__variant_combinations
