@@ -30,7 +30,8 @@ import reframe.utility.osext as osext
 import reframe.utility.typecheck as typ
 
 
-from reframe.frontend.distribute import distribute_tests, getallnodes
+from reframe.frontend.testgenerators import (distribute_tests,
+                                             getallnodes, repeat_tests)
 from reframe.frontend.executors.policies import (SerialExecutionPolicy,
                                                  AsynchronousExecutionPolicy)
 from reframe.frontend.executors import Runner, generate_testcases
@@ -46,6 +47,7 @@ def format_env(envvars):
     return ret
 
 
+@logging.time_function
 def list_checks(testcases, printer, detailed=False, concretized=False):
     printer.info('[List of matched checks]')
     unique_checks = set()
@@ -113,6 +115,7 @@ def list_checks(testcases, printer, detailed=False, concretized=False):
         printer.info(f'Found {len(unique_checks)} check(s)\n')
 
 
+@logging.time_function
 def describe_checks(testcases, printer):
     records = []
     unique_names = set()
@@ -185,6 +188,7 @@ def calc_verbosity(site_config, quiesce):
     return curr_verbosity - quiesce
 
 
+@logging.time_function_noexit
 def main():
     # Setup command line options
     argparser = argparse.ArgumentParser()
@@ -209,6 +213,11 @@ def main():
     misc_options = argparser.add_argument_group('Miscellaneous options')
 
     # Output directory options
+    output_options.add_argument(
+        '--compress-report', action='store_true',
+        help='Compress the run report file',
+        envvar='RFM_COMPRESS_REPORT', configvar='general/compress_report'
+    )
     output_options.add_argument(
         '--dont-restage', action='store_false', dest='clean_stagedir',
         help='Reuse the test stage directory',
@@ -314,7 +323,7 @@ def main():
     # partition environments as the runtime is created, similarly to how the
     # system partitions are treated. Currently, this facilitates the
     # implementation of fixtures, but we should reconsider it: see discussion
-    # in https://github.com/eth-cscs/reframe/issues/2245
+    # in https://github.com/reframe-hpc/reframe/issues/2245
     select_options.add_argument(
         '-p', '--prgenv', action='append', default=[r'.*'],  metavar='PATTERN',
         configvar='general/valid_env_names',
@@ -408,6 +417,10 @@ def main():
     run_options.add_argument(
         '--mode', action='store', help='Execution mode to use'
     )
+    run_options.add_argument(
+        '--repeat', action='store', metavar='N',
+        help='Repeat selected tests N times'
+    )
     run_options.add_argument(
         '--restore-session', action='store', nargs='?', const='',
         metavar='REPORT',
@@ -940,6 +953,8 @@ def main():
     print_infoline('output directory', repr(session_info['prefix_output']))
     printer.info('')
     try:
+        logging.getprofiler().enter_region('test processing')
+
         # Need to parse the cli options before loading the tests
         parsed_job_options = []
         for opt in options.job_options:
@@ -1035,6 +1050,20 @@ def main():
                 f'{len(testcases)} remaining'
             )
 
+        if options.repeat is not None:
+            try:
+                num_repeats = int(options.repeat)
+                if num_repeats <= 0:
+                    raise ValueError
+            except ValueError:
+                raise errors.CommandLineError(
+                    "argument to '--repeat' option must be "
+                    "a non-negative integer"
+                ) from None
+
+            testcases = repeat_tests(testcases, num_repeats)
+            testcases_all = testcases
+
         if options.distribute:
             node_map = getallnodes(options.distribute, parsed_job_options)
 
@@ -1227,7 +1256,7 @@ def main():
             errmsg = "invalid option for --flex-alloc-nodes: '{0}'"
             sched_flex_alloc_nodes = int(options.flex_alloc_nodes)
             if sched_flex_alloc_nodes <= 0:
-                raise errors.ConfigError(
+                raise errors.CommandLineError(
                     errmsg.format(options.flex_alloc_nodes)
                 )
         except ValueError:
@@ -1236,7 +1265,7 @@ def main():
         exec_policy.sched_flex_alloc_nodes = sched_flex_alloc_nodes
         exec_policy.sched_options = parsed_job_options
         if options.maxfail < 0:
-            raise errors.ConfigError(
+            raise errors.CommandLineError(
                 f'--maxfail should be a non-negative integer: '
                 f'{options.maxfail!r}'
             )
@@ -1299,8 +1328,11 @@ def main():
             report_file = runreport.next_report_filename(report_file)
             try:
                 with open(report_file, 'w') as fp:
-                    jsonext.dump(json_report, fp, indent=2)
-                    fp.write('\n')
+                    if rt.get_option('general/0/compress_report'):
+                        jsonext.dump(json_report, fp)
+                    else:
+                        jsonext.dump(json_report, fp, indent=2)
+                        fp.write('\n')
 
                 printer.info(f'Run report saved in {report_file!r}')
             except OSError as e:
@@ -1342,6 +1374,7 @@ def main():
         sys.exit(1)
     finally:
         try:
+            logging.getprofiler().exit_region()     # region: 'test processing'
             log_files = logging.log_files()
             if site_config.get('general/0/save_log_files'):
                 log_files = logging.save_log_files(rt.output_prefix)
@@ -1352,3 +1385,6 @@ def main():
         finally:
             if not restrict_logging():
                 printer.info(logfiles_message())
+
+            logging.getprofiler().exit_region()     # region: 'main'
+            logging.getprofiler().print_report(printer.debug)
