@@ -1,4 +1,4 @@
-# Copyright 2016-2021 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2022 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -13,9 +13,10 @@ import sys
 
 import reframe.utility as util
 from reframe.core.exceptions import DependencyError
-from reframe.core.logging import getlogger
+from reframe.core.logging import getlogger, time_function
 
 
+@time_function
 def build_deps(cases, default_cases=None):
     '''Build dependency graph from test cases.
 
@@ -31,7 +32,7 @@ def build_deps(cases, default_cases=None):
 
         ret = {}
         for c in cases:
-            cname = c.check.name
+            cname = c.check.unique_name
             ret.setdefault(cname, [])
             ret[cname].append(c)
 
@@ -130,15 +131,16 @@ def _reduce_deps(graph):
     '''Reduce test case graph to a test-only graph.'''
     ret = {}
     for case, deps in graph.items():
-        test_deps = util.OrderedSet(d.check.name for d in deps)
+        test_deps = util.OrderedSet(d.check.unique_name for d in deps)
         try:
-            ret[case.check.name] |= test_deps
+            ret[case.check.unique_name] |= test_deps
         except KeyError:
-            ret[case.check.name] = test_deps
+            ret[case.check.unique_name] = test_deps
 
     return ret
 
 
+@time_function
 def validate_deps(graph):
     '''Validate dependency graph.'''
 
@@ -181,6 +183,7 @@ def validate_deps(graph):
         sources -= visited
 
 
+@time_function
 def prune_deps(graph, testcases, max_depth=None):
     '''Prune the graph so that it contains only the specified cases and their
     dependencies up to max_depth.
@@ -205,6 +208,7 @@ def prune_deps(graph, testcases, max_depth=None):
     return pruned_graph
 
 
+@time_function
 def toposort(graph, is_subgraph=False):
     '''Return a list of the graph nodes topologically sorted.
 
@@ -251,11 +255,11 @@ def toposort(graph, is_subgraph=False):
     # Index test cases by test name
     cases_by_name = {}
     for c in graph.keys():
-        c.level = levels[c.check.name]
+        c.level = levels[c.check.unique_name]
         try:
-            cases_by_name[c.check.name].append(c)
+            cases_by_name[c.check.unique_name].append(c)
         except KeyError:
-            cases_by_name[c.check.name] = [c]
+            cases_by_name[c.check.unique_name] = [c]
 
     return list(itertools.chain(*(retrieve(cases_by_name, n, [])
                                   for n in visited)))
