@@ -110,6 +110,7 @@ class TestStats:
                     'environment': None,
                     'fail_phase': None,
                     'fail_reason': None,
+                    'fixture': check.is_fixture(),
                     'jobid': None,
                     'job_stderr': None,
                     'job_stdout': None,
@@ -190,6 +191,22 @@ class TestStats:
                             'value': val
                         })
 
+                # Add any loggable variables and parameters
+                entry['check_vars'] = {}
+                test_cls = type(check)
+                for name, var in test_cls.var_space.items():
+                    if var.is_loggable():
+                        try:
+                            entry['check_vars'][name] = getattr(check, name)
+                        except AttributeError:
+                            entry['check_vars'][name] = '<undefined>'
+
+                entry['check_params'] = {}
+                test_cls = type(check)
+                for name, param in test_cls.param_space.items():
+                    if param.is_loggable():
+                        entry['check_params'][name] = getattr(check, name)
+
                 testcases.append(entry)
 
             self._run_data.append({
@@ -234,7 +251,7 @@ class TestStats:
                          f"{r['dependencies_actual']}")
             printer.info(f"  * Maintainers: {r['maintainers']}")
             printer.info(f"  * Failing phase: {r['fail_phase']}")
-            if rerun_info:
+            if rerun_info and not r['fixture']:
                 if rt.runtime().get_option('general/0/compact_test_names'):
                     cls = r['display_name'].split(' ')[0]
                     variant = r['unique_name'].replace(cls, '')
