@@ -678,7 +678,7 @@ def find_modules(substr, environ_mapping=None):
     for p in current_system.partitions:
         for e in p.environs:
             rt.loadenv(p.local_env, e)
-            modules = ms.available_modules(substr)
+            modules = OrderedSet(ms.available_modules(substr))
             snap0.restore()
             for m in modules:
                 if _is_valid_for_env(m, e.name):
@@ -925,6 +925,23 @@ def nodelist_abbrev(nodes):
     return ','.join(str(ng) for ng in node_groups)
 
 
+class temp_setattr:
+    '''Context manager to temporarily change the attribute value of an
+    object.'''
+
+    def __init__(self, obj, attr, val):
+        self._obj = obj
+        self._attr = attr
+        self._newval = val
+        self._saved = getattr(obj, attr)
+
+    def __enter__(self):
+        setattr(self._obj, self._attr, self._newval)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        setattr(self._obj, self._attr, self._saved)
+
+
 class ScopedDict(UserDict):
     '''This is a special dictionary that imposes scopes on its keys.
 
